<?php

/**
 * Display the html content for our custom post meta box
 **/
function iwacontact_box() {
	// Use nonce for verification
	wp_nonce_field( plugin_basename(__FILE__), 'iwajax_contact' );
	// Get the custom fields for this post
	$fields = get_post_custom();
	// Check if the custom fields contain any values for our post meta box fields
	$iwacontact_data = ( key_exists( 'iwacontact_data', $fields ) ) ? $fields['iwacontact_data'][0] : '';
	$iwacf_sendto = ( key_exists( 'iwacontact_sendto', $fields ) ) ? $fields['iwacontact_sendto'][0] : 'you@yourdomain.com';
	$iwacf_subject = ( key_exists( 'iwacontact_subject', $fields ) ) ? $fields['iwacontact_subject'][0] : 'New contact form submission!';
	$iwacf_from = ( key_exists( 'iwacontact_from', $fields ) ) ? $fields['iwacontact_from'][0] : 'no-reply@yourdomain.com';
	$iwacf_submit_value = ( key_exists( 'iwacontact_submit_value', $fields ) ) ? $fields['iwacontact_submit_value'][0] : 'Send Enquiry';
	// Display the first part of our custom meta box
	?>
	<input type="hidden" id="iwacontact_data" name="iwacontact_data" value="<?php echo $iwacontact_data; ?>" />
	<h2 style="margin: 0 5px;">Form Configuration</h2>
	<p class="config">
		<label for="iwacf_sendto" style="font-weight: bold;">Send Submissions To</label>
		<input type="text" id="iwacf_sendto" name="iwacf_sendto" value="<?php echo $iwacf_sendto; ?>" />
	</p>
	<p class="config">
		<label for="iwacf_subject" style="font-weight: bold;">Subject Line</label>
		<input type="text" id="iwacf_subject" name="iwacf_subject" value="<?php echo $iwacf_subject; ?>" />
	</p>
	<p class="config">
		<label for="iwacf_from" style="font-weight: bold;">From Address</label>
		<input type="text" id="iwacf_from" name="iwacf_from" value="<?php echo $iwacf_from; ?>" />
	</p>
	<p class="config">
		<label for="iwacf_submit_value" style="font-weight: bold;">Submit Button Text</label>
		<input type="text" id="iwacf_submit_value" name="iwacf_submit_value" value="<?php echo $iwacf_submit_value; ?>" />
	</p>
	<p class="shortcode">
		<label for="iwacf_shortcode" style="font-weight: bold;">Shortcode<em>Copy &amp; paste in to any post or page!</em></label>
		<input type="text" id="iwacf_shortcode" value="[insert_ajaxcontact id=<?php the_ID(); ?>]" <?php echo $iwacf_submit_value; ?>" readonly="readonly" />
	</p>
	<div id="formFields">
		<h2 class="fields-title">
			Form Fields
			<input type="button" value="Add Field" class="addFieldButton rightbutton" />
		</h2>
		<?php
		// Now we need to display the form fields if there where any
		// So, if our iwacontact_data custom field has a value containing ';;'
		if ( preg_match( '/\;\;/', $iwacontact_data ) ) {
			// Find each field by splitting the custom field contents on ';;'
			$form_fields = preg_split( '/\;\;/', $iwacontact_data );
			// Create a new array for the correctly ordered form fields
			$ordered_fields = array();
			// For each form field
			foreach ( $form_fields as $field ) {
				// Find the field attributes by splitting the string on '::'
				$field_attrs = preg_split( '/\:\:/', $field );
				// Add them to our ordered fields array
				array_push( $ordered_fields, $field_attrs );
			}
			// Sort the ordered fields array by the specified display order
			usort( $ordered_fields, "sort_fields" );
			$count = 0;
			$form_fields_html = '';
			$field_display_html = '';
			// For each field in our correctly ordered form field array
			foreach ( $ordered_fields as $field ) {
				// Place the field attributes in an easier to use associative array
				$new_field_attrs = array(
					'displayorder' => $field[1],
					'fieldtype' => $field[2],
					'fieldname' => $field[3],
					'fieldoptions' => $field[4],
					'fieldrequired' => $field[5],
					'fieldvalidation' => $field[6]
				);
				if ( key_exists( 7, $field ) )
					$new_field_attrs = array_merge( $new_field_attrs, array( 'fieldlabel' => $field[7] ) );
				// Echo the field editor html for the current field
				$form_fields_html .= get_new_contact_field( $count, $new_field_attrs );
				$field_display_html .= get_new_contact_display( $count, $new_field_attrs );
				$count++;
			}
			echo '<ul id="fieldDisplays">' . $field_display_html . '</ul>';
			echo $form_fields_html;
		}
		else {
			// No contact fields exist yet, just display an empty one
			echo '<ul id="fieldDisplays">' . get_new_contact_display( '0', array( 'displayorder' => 1 ) ) . '</ul>';
			echo get_new_contact_field( '0', array( 'displayorder' => 1 ) );
		}
		// Now we need to have a hidden blank field for jQuery to use
		// as a template to auto-magically add new fields to the page
		?>
	</div>
	<div id="newFormField" style="display: none;">
		<?php
			echo get_new_contact_display( 'x' );
			echo get_new_contact_field( 'x' );
		?>
	</div>
	<div class="iwacf-tooltip tooltip-default">
		<p>Enter the default value for this field or leave empty for no default value.</p>
		<p>You may also use [[the_title]] or [[the_id]] to include the current post/page title or ID in the default field value.</p>
	</div>
	<div class="iwacf-tooltip tooltip-options">
		<p>Enter the options the user will be asked to select from, one option per line.</p>
		<p>You may also use [[the_title]] or [[the_id]] to include the current post/page title or ID in an option.</p>
	</div>
    <p style="margin: 0 5px;">*Please Note:  These forms only work as shortcodes and can not be "viewed as pages" using link above after saving or adding.</p>
	<?php
	
}

/**
 * Add our meta box to the add/edit page screen
 **/
function iwacontact_add_meta() {
//	add_meta_box(
//		'iwajax_contact', // HTML 'id' attribute of the edit screen section
//		'AJAX Contact', // Title of the edit screen section, visible to user
//		'iwacontact_box', // Function that prints out the HTML for the edit screen section.
//		'page', // The type of Write screen on which to show the edit screen section
//		'normal', // The part of the page where the edit screen section should be shown
//		'high' // The priority within the context where the boxes should show
//	);
	add_meta_box(
		'iwajax_contact', // HTML 'id' attribute of the edit screen section
		'Form Editor', // Title of the edit screen section, visible to user
		'iwacontact_box', // Function that prints out the HTML for the edit screen section.
		'iwacontactform', // The type of Write screen on which to show the edit screen section
		'normal', // The part of the page where the edit screen section should be shown
		'high' // The priority within the context where the boxes should show
	);
}

add_action( 'add_meta_boxes', 'iwacontact_add_meta' );

/**
 * Save the iwacontact meta box data on save_post action
 *
 * @param integer $post_id The post ID
 **/
function iwacontact_save_meta( $post_id ) {
	// If our hidden field wasn't in the submitted data, return (don't save)
	if ( !key_exists( 'iwajax_contact', $_POST ) )
		return $post_id;
	// If the nonce couldn't be verified, return (don't save)
	if ( !wp_verify_nonce( $_POST['iwajax_contact'], plugin_basename(__FILE__) ) )
		return $post_id;
	// If doing an autosave, return (don't save)
	if ( defined('DOING_AUTOSAVE') && DOING_AUTOSAVE ) 
		return $post_id;
	// If the post_type is not a page, return (don't save)
	if ( 'page' == $_POST['post_type'] ) {
		if ( !current_user_can( 'edit_page', $post_id ) )
			return $post_id;
	}
	$form_fields = array(); // Form fields associative array
	$field_names = array(); // Field names array
	// For each item in the $_POST array
	foreach ( $_POST as $key => $value ) {
		// Using a regex, check if it's one of our attribute fields (e.g. iwacf_fieldname_1)
		if ( preg_match( '/^iwacf\_[a-z]+\_[0-9]+$/', $key ) ) {
			// Split the array key on _ (e.g. ( [0] => iwacf, [1] => fieldname, [2] => 1)
			$attrs = preg_split( '/\_/', $key );
			// Assign this attribute field's serial number to $field_no
			$field_no = $attrs[2];
			// If this is a fieldname attribute, add the value of it to the $field_names array
			if ( $attrs[1] == 'fieldname' )
				array_push( $field_names, $value );
			// Create a new associative array containing the attribute name => value
			$array_item = array( $attrs[1] => $value );
			// If the $form_fields associative array does not contain an array with this field number
			if ( !key_exists( $field_no, $form_fields ) || !is_array( $form_fields[$field_no] ) )
				$form_fields[$field_no] = array(); // Add it to $form_fields as an empty array
			// Merge this attribute name=>value pair on to the $form_fields array
			$form_fields[$field_no] = array_merge( $form_fields[$field_no], $array_item );
		}
	}
	// Now $form_fields is populated with all our attribute fields like;
	// array( [0] => array( 'fieldname' => 'My First Field', 'fieldtype' => 'input' ) ,
	//   [1] => array( 'fieldname' => 'My Second Field', 'fieldtype' => 'selectbox' ) )
	$iwacontact_data = array();
	$iwacontact_data_str = '';
	$count = 0;
	// For each associative array in our $form_fields array
	foreach ( $form_fields as $field ) {
		// Increase the counter
		$count++;
		// Make the field name safe to be turned in to a html ID/name attribute
		$safe_field_name = mb_strtolower( trim( $field['fieldname'] ) ); // Clean leading/trailing white space and make the string lower case
		$safe_field_name = preg_replace( '/\ /', '_', $safe_field_name ); // Replace any spaces with underscores (_)
		$safe_field_name = preg_replace( "/[^a-zA-Z0-9-_]/", "", $safe_field_name ); // Remove any non alphanumeric characters
		// Set some defaults in case anything is left empty
		$displayorder = ( key_exists( 'displayorder', $field ) ) ? $field['displayorder'] : $count;
		$fieldoptions = ( key_exists( 'fieldoptions', $field ) ) ? $field['fieldoptions'] : '';
		$fieldrequired = ( key_exists( 'fieldrequired', $field ) ) ? $field['fieldrequired'] : '';
		// Turn this field's attributes in to an array
		$field_data = array(
			$safe_field_name,
			$displayorder,
			$field['fieldtype'],
			$field['fieldname'],
			$fieldoptions,
			$fieldrequired,
			$field['fieldvalidation'],
			$field['fieldlabel']
		);
		// Turn the array in to a '::' separated string
		$field = join( '::', $field_data );
		// Add the field on to the $iwacontact_data array
		array_push( $iwacontact_data, $field );
	}
	// Turn the fields in $iwacontact_data in to a ';;' separated string
	$iwacontact_data_str = join( ';;', $iwacontact_data );
	// Update all the post custom fields with our meta data
	update_post_meta( $post_id, 'iwacontact_data', $iwacontact_data_str );
	update_post_meta( $post_id, 'iwacontact_sendto', trim( $_POST['iwacf_sendto'] ) );
	update_post_meta( $post_id, 'iwacontact_subject', trim( $_POST['iwacf_subject'] ) );
	update_post_meta( $post_id, 'iwacontact_from', trim( $_POST['iwacf_from'] ) );
	update_post_meta( $post_id, 'iwacontact_submit_value', trim( $_POST['iwacf_submit_value'] ) );
}

add_action( 'save_post', 'iwacontact_save_meta' );

/**
 * Return a new field in the admin edit contact form box
 *
 * @param integer $sequence The sequential number for this contact field
 * @param array $data The data to pre-populate this field with (if any)
 **/
function get_new_contact_display( $sequence, $data = array() ) {
	// If any necessary fields are empty, fill them with the default value
	$fieldname = ( key_exists( 'fieldname', $data ) ) ? $data['fieldname'] : 'New Field';
	$fieldtype = ( key_exists( 'fieldtype', $data ) ) ? $data['fieldtype'] : 'input';
	// Html for a new contact form field display
	return "<li id='fieldDisplay_$sequence' class='field-display icon-$fieldtype'>"
		. "<a href='#'>$fieldname</a>"
		. '</li>';
}

/**
 * Return a new *hidden* field editor dialog in the admin edit contact form box
 *
 * @param integer $sequence The sequential number for this contact field
 * @param array $data The data to pre-populate this field with (if any)
 **/
function get_new_contact_field( $sequence, $data = array() ) {
	// If any necessary fields are empty, fill them with the default value
	$fieldname = ( key_exists( 'fieldname', $data ) ) ? $data['fieldname'] : 'New Field';
	$fieldtype = ( key_exists( 'fieldtype', $data ) ) ? $data['fieldtype'] : 'input';
	$fieldoptions = ( key_exists( 'fieldoptions', $data ) ) ? $data['fieldoptions'] : '';
	$displayorder = ( key_exists( 'displayorder', $data ) ) ? $data['displayorder'] : '';
	$fieldrequired = ( key_exists( 'fieldrequired', $data ) ) ? $data['fieldrequired'] : '';
	$fieldvalidation = ( key_exists( 'fieldvalidation', $data ) ) ? $data['fieldvalidation'] : 'none';
	$fieldlabel = ( key_exists( 'fieldlabel', $data ) ) ? $data['fieldlabel'] : '1';
	$default_label_label = 'Default Value';
	switch ( $field_type ) {
		case 'input': //break;
		case 'password': //break;
		case 'textarea': //break;
		case 'hidden': //break;
		case 'readonly':
			$default_label_label = 'Default Value <span class="iwacf-help help-default"></span>';
			break;
		case 'radio': //break;
		case 'selectbox': //break;
		case 'multiselect':
			$default_label_label = 'Options (1 per line) <span class="iwacf-help help-options"></span>';
			break;
		case 'checkbox': //break;
		case 'sendcopy': //break;
		case 'h1': //break;
		case 'h2': //break;
		case 'h3': //break;
		case 'h4':
		default:
			$default_label_label = 'Default Value <span class="iwacf-help help-default"></span>';
			break;
	}
	// Html for a new contact form field
	return "<div id='formField_$sequence' class='form-field hidden'>"
		. '<h2>Field Editor</h2>'
		. '<p>'
		. "<label for='iwacf_fieldname_$sequence'>Field Title</label>"
		. "<input id='iwacf_fieldname_$sequence' name='iwacf_fieldname_$sequence' type='text' value='$fieldname' class='field-title widefat' />"
		. '</p>'
		. '<p>'
		. "<label for='iwacf_fieldtype_$sequence'>Field Type</label>"
		. "<select id='iwacf_fieldtype_$sequence' name='iwacf_fieldtype_$sequence' class='field-type widefat'>"
		. '<option value="input" ' . ( ( $fieldtype == 'input' ) ? 'selected="selected"' : '' ) . '>Text Input</option>'
		. '<option value="textarea" ' . ( ( $fieldtype == 'textarea' ) ? 'selected="selected"' : '' ) . '>Multi-Line Text Input</option>'
		. '<option value="selectbox" ' . ( ( $fieldtype == 'selectbox' ) ? 'selected="selected"' : '' ) . '>Select Box</option>'
		. '<option value="multiselect" ' . ( ( $fieldtype == 'multiselect' ) ? 'selected="selected"' : '' ) . '>Multiple Selection Box</option>'
		. '<option value="checkbox" ' . ( ( $fieldtype == 'checkbox' ) ? 'selected="selected"' : '' ) . '>Check Box</option>'
		. '<option value="radio" ' . ( ( $fieldtype == 'radio' ) ? 'selected="selected"' : '' ) . '>Radio Buttons</option>'
		. '<option value="password" ' . ( ( $fieldtype == 'password' ) ? 'selected="selected"' : '' ) . '>Password Input</option>'
		. '<option value="hidden" ' . ( ( $fieldtype == 'hidden' ) ? 'selected="selected"' : '' ) . '>Hidden Field</option>'
		. '<option value="readonly" ' . ( ( $fieldtype == 'readonly' ) ? 'selected="selected"' : '' ) . '>Read-Only Input</option>'
		// Sorry, sendcopy didn't make this release
		//. '<option value="sendcopy" ' . ( ( $fieldtype == 'sendcopy' ) ? 'selected="selected"' : '' ) . '>Send Copy Check Box</option>'
		. '<option value="h1" ' . ( ( $fieldtype == 'h1' ) ? 'selected="selected"' : '' ) . '>Heading 1</option>'
		. '<option value="h2" ' . ( ( $fieldtype == 'h2' ) ? 'selected="selected"' : '' ) . '>Heading 2</option>'
		. '<option value="h3" ' . ( ( $fieldtype == 'h3' ) ? 'selected="selected"' : '' ) . '>Heading 3</option>'
		. '<option value="h4" ' . ( ( $fieldtype == 'h4' ) ? 'selected="selected"' : '' ) . '>Heading 4</option>'
		. '</select>'
		. '</p>'
		. '<p class="iwacf_fieldoptions">'
		. "<label for='iwacf_fieldoptions_$sequence'>$default_label_label</label>"
		. "<textarea id='iwacf_fieldoptions_$sequence' name='iwacf_fieldoptions_$sequence' rows='3' class='widefat'>$fieldoptions</textarea>"
		. '</p>'
		. '<p>'
		. "<label for='iwacf_displayorder_$sequence'>Display Order</label>"
		. "<input id='iwacf_displayorder_$sequence' name='iwacf_displayorder_$sequence' type='text' value='$displayorder' class='widefat' />"
		. '</p>'
		. '<p>'
		. "<label for='iwacf_fieldvalidation_$sequence'>Validation</label>"
		. "<select id='iwacf_fieldvalidation_$sequence' name='iwacf_fieldvalidation_$sequence' class='field-validation widefat'>"
		. '<option value="none" ' . ( ( $fieldvalidation == 'none' ) ? 'selected="selected"' : '' ) . '>No Validation</option>'
		. '<option value="email" ' . ( ( $fieldvalidation == 'email' ) ? 'selected="selected"' : '' ) . '>Email Address Validation</option>'
		. '</select>'
		. '</p>'
		. '<p class="border-bottom border-top">'
		. "<input type='checkbox' id='iwacf_fieldrequired_$sequence' name='iwacf_fieldrequired_$sequence' value='1' " . ( ( $fieldrequired == '1' ) ? 'checked="checked"' : '' ) . ' class="iwacf-checkbox" />'
		. "<label for='iwacf_fieldrequired_$sequence'>This field is required</label>"
		. '</p>'
		. '<p class="border-bottom">'
		. "<input type='checkbox' id='iwacf_fieldlabel_$sequence' name='iwacf_fieldlabel_$sequence' value='1' " . ( ( $fieldlabel == '1' ) ? 'checked="checked"' : '' ) . ' class="iwacf-checkbox" />'
		. "<label for='iwacf_fieldlabel_$sequence'>Show a label for this field</label>"
		. '</p>'
		. '<p>'
		. "<input type='button' id='removeField_$sequence' class='deleteFieldButton' value='Delete' />"
		. '<input type="button" value="Add New Field" class="addFieldButton" />'
		. '</p>'
		. '</div>';
}
